﻿/*******************************************************************************
 * Copyright 2009 Seapine Software, Inc. All rights reserved.
 * 
 * IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. (“Seapine”)
 * in consideration of your agreement to the following terms, and your use, installation,
 * modification or redistribution of this Software constitutes acceptance of these terms.
 * If you do not agree with these terms, please do not use, install, modify or redistribute
 * this Software.
 * 
 * In consideration of your agreement to abide by the following terms, and subject to these
 * terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
 * this original Seapine software (the “Software”), to use, reproduce, modify and redistribute
 * the Software, with or without modifications, in source and/or binary forms; provided that
 * if you redistribute the Software in its entirety and without modifications, you must retain
 * this notice and the following text and disclaimers in all such redistributions of the Software.
 * Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
 * or promote products derived from the Software without specific prior written permission from
 * Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
 * implied, are granted by Seapine herein, including but not limited to any patent rights that
 * may be infringed by your derivative works or by other works in which the Software may be
 * incorporated.
 * 
 * The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
 * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
 * OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS. 
 * 
 * IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
 * REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
 * UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
 * IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/



using System;
using System.Diagnostics;
using System.Windows.Forms;
using Microsoft.TeamFoundation.VersionControl.Client;
using System.Collections.Generic;
using System.IO;
using System.Text;

[Serializable]
public class CheckForCommentsPolicy : PolicyBase
{
    public override string Description
    {
        get { return "Remind users to include a TestTrack defect number with each check-in"; }

    }

    // This is a string that is stored with the policy definition on the source
    // control server. If a user does not have the policy plug-in installed, this string
    // is displayed.  You can use this to explain to the user how they should 
            // install the policy plug-in.
    public override string InstallationInstructions
    {
        get { return "To install this policy, read InstallInstructions.txt."; }
    }

    // This string identifies the type of policy. It is displayed in the 
 // policy list when you add a new policy to a Team Project.
    public override string Type
    {
        get { return "Check for Comments Policy"; }
    }

    // This string is a description of the type of policy. It is displayed 
    // when you select the policy in the Add Check-in Policy dialog box.
    public override string TypeDescription
    {
        get { return "This policy will prompt the user to decide whether or not they should be allowed to check in."; }
    }

    // This method is called by the policy framework when you create 
    // a new check-in policy or edit an existing check-in policy.
    // You can use this to display a UI specific to this policy type 
    // allowing the user to change the parameters of the policy.
    public override bool Edit(IPolicyEditArgs args)
    {
        // Do not need any custom configuration
        return true;
    }

    // This method performs the actual policy evaluation. 
    // It is called by the policy framework at various points in time
    // when policy should be evaluated. In this example, the method 
    // is invoked when various asyc events occur that may have 
    // invalidated the current list of failures.
    public override PolicyFailure[] Evaluate()
    {
        
        string proposedComment = PendingCheckin.PendingChanges.Comment;
        //if comments are not provided, return a policy failure with message
        if (String.IsNullOrEmpty(proposedComment))
        {
            return new PolicyFailure[] {
                new PolicyFailure("Please provide a TestTrack defect number in the format <:#:>", this) };
        }
        //If comments are provided and appears to contain the token, proceed 
        else if (proposedComment.Contains("<:") && proposedComment.Contains(":>"))
        {
            long iDNum = 0;
            string sDNum = "";
            //extract the contents in the token
            proposedComment = proposedComment.Remove(0, proposedComment.IndexOf("<:") + 2);
            sDNum = proposedComment.Substring(0, proposedComment.IndexOf(":>"));
            bool isInt = false;
            //try to convert to int to make sure a valid number is provided
            try
            {
                iDNum = Convert.ToInt64(sDNum);
                //if no exception, then we set the boolean to true
                isInt = true;
            }
            catch (Exception d)
            {
                //if we get an exception, we set the boolean to false
                isInt = false;
            }
            //if the token does not have anything or can not convert it to int, return policy failure
            if ((sDNum == "") || (sDNum == null) || (isInt == false))
            {
                return new PolicyFailure[] {
                new PolicyFailure("Please provide a TestTrack defect number in the format <:#:>", this) };

            }
            else
            {             
                //attach each pending change to the defect
                foreach (PendingChange pChange in PendingCheckin.PendingChanges.AllPendingChanges)
                {
                    AttachToDefect(pChange.ServerItem,iDNum,pChange.Version,pChange.CreationDate);

                }
                return new PolicyFailure[0];
            }
        }
        else
        {
            return new PolicyFailure[] {
                new PolicyFailure("Please provide a TestTrack defect number in the format <:#:>", this) };

        }
    }

    // This method is called if the user double-clicks on 
    // a policy failure in the UI. In this case a message telling the user 
    // to supply some comments is displayed.
    public override void Activate(PolicyFailure failure)
    {
        MessageBox.Show("Please provide a TestTrack defect number in the format <:#:>", "How to fix your policy failure");
    }

    // This method is called if the user presses F1 when a policy failure 
    // is active in the UI. In this example, a message box is displayed.
    public override void DisplayHelp(PolicyFailure failure)
    {
        MessageBox.Show("This policy helps you to remember to a TestTrack defect number with your check-ins.", "Prompt Policy Help");
    }
    private void AttachToDefect(string file,long dNum, int version, DateTime date)
    {
        string sccFile = file;
        string scmVersion = version.ToString();
        
        string errMessage = "";

        string m_User = "Administrator";
        string m_Password = "";
        string m_Url = "http://localhost/scripts/ttsoapcgi.exe";
        string m_ProjectName = "Sample Project";
        long ttCookie = 0;



        ttsoapcgi m_ttsoapcgi = new ttsoapcgi();
        m_ttsoapcgi.Url = m_Url;

        CProject m_Project = new CProject();
        CProject[] m_Projects = m_ttsoapcgi.getProjectList(m_User, m_Password);
        for (int i = 0; i < m_Projects.Length; i++)
        {
            if (m_Projects[i].database.name == m_ProjectName)
            {
                m_Project = m_Projects[i];
            }
        }

        try
        {
            ttCookie = m_ttsoapcgi.ProjectLogon(m_Project, m_User, m_Password);
        }
        catch (Exception fcc)
        {
            errMessage = fcc.Message;
        }
        if (errMessage != "")
        {

        }
        else
        {
            CDefect m_Defect = m_ttsoapcgi.editDefect(ttCookie, dNum, "", false);
            CSCCFileRecord m_SCCFile = new CSCCFileRecord();
            m_SCCFile.mstrFileName = sccFile;
            m_SCCFile.mstrFixedRevision = scmVersion;
            m_SCCFile.mdateFixedTimestamp = date;
            if (m_Defect.pSCCFileList == null)
            {
                m_Defect.pSCCFileList = new CSCCFileRecord[1];
                m_Defect.pSCCFileList.SetValue(m_SCCFile, m_Defect.pSCCFileList.Length - 1);
            }
            else
            {
                try
                {
                    CSCCFileRecord[] sccFileList = new CSCCFileRecord[m_Defect.pSCCFileList.Length + 1];
                    for (int i = 0; i < m_Defect.pSCCFileList.Length; i++)
                    {
                        sccFileList.SetValue(m_Defect.pSCCFileList[i], i);
                    }

                    sccFileList.SetValue(m_SCCFile, m_Defect.pSCCFileList.Length);
                    m_Defect.pSCCFileList = sccFileList;
                }
                catch (Exception f)
                {
                    logger(f.Message);
                }
            }
            m_ttsoapcgi.saveDefect(ttCookie, m_Defect);

            m_ttsoapcgi.DatabaseLogoff(ttCookie);
        }





    }
    private void logger(String log)
    {
        StreamWriter m_stream = new StreamWriter("c:\\script.log", true);

        // Prepend the current date/time to each line of output.
        m_stream.WriteLine(log);
        m_stream.Flush();
        m_stream.Close();

    }
}